//
//  Customer.swift
//  IndexedTableViewDemo
//
//  Created by Kevin McNeish on 2/14/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import Foundation
import CoreData

class Customer<T:CustomerEntity>: ABusinessObject<T> {
    
    var entityList = Array<T>()
    
    var allKeys = ["A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","#"]
    
    var formatter = NSNumberFormatter()
    var objectsByKey = Dictionary<String, Array<CustomerEntity>>()
    var activeKeys = Array<String>()
    var indexedSectionsCount: Int {
        return self.activeKeys.count
    }
    
    override init() {
        super.init()
        self.createIndex()
    }
    
    override func getAllEntities() -> Array<T> {
        self.entityList = super.getAllEntities()
        self.createIndex()
        return self.entityList
    }
    
    func createIndex() {
        
        var dictionary = Dictionary<String, Array<CustomerEntity>>()
        var arrayOfCustomersWithNames : Array<CustomerEntity>?
        var arrayOfCustomersWithNumbers = Array<CustomerEntity>()
        var firstLetter: String
        
        self.objectsByKey = Dictionary<String, Array<CustomerEntity>>()
        
        // Group customers with the same first letter of the last name
        // into arrays
        for customer in self.entityList {
            
            // Get the first letter in the last name
            let index = advance(customer.lastName.startIndex, 1)
            firstLetter = customer.lastName.substringToIndex(index)
            
            // Check if this is a number
            if let letter = self.formatter.numberFromString(firstLetter) {
                // It is, add it to the number list
                arrayOfCustomersWithNumbers.append(customer)
            }
            else {
                // It's not. See if we've come across this letter yet
                arrayOfCustomersWithNames = dictionary[firstLetter]
                
                if arrayOfCustomersWithNames == nil {
                    // No, create an array to hold items starting with this letter
                    arrayOfCustomersWithNames = Array<T>()
                    
                }
                // Add the object to the current array of names
                arrayOfCustomersWithNames!.append(customer)
                // Store the array in the dictionary
                dictionary[firstLetter] = arrayOfCustomersWithNames
            }
        }
        
        // Sort the first letters alphabetically into a new array
        var keyArray = Array(dictionary.keys).sorted(<)
        
        // Create a copy of the array
        self.activeKeys = keyArray
        
        // If any items starting with numbers were encountered,
        // add the array to the dictionary and # to the keys
        if arrayOfCustomersWithNumbers.count > 0 {
            dictionary["#"] = arrayOfCustomersWithNumbers
        }
        
        // Sort arrays in the dictionary alphabetically, then add the
        // sorted arrays to the objectsByKey dictionary
        for key in dictionary.keys {
            
            var array = dictionary[key]
            array?.sort({ $0.lastName < $1.lastName })
            
            self.objectsByKey[key] = array
        }
    }
    
    // Returns the number of entity objects for the specified key
    func objectCountForKey(index: Int) -> Int {
        
        var key = self.activeKeys[index]
        var namesArray = self.objectsByKey[key]
        return namesArray!.count
    }
    
    // Returns the entity object at the specified index path
    func objectAtIndexPath(indexPath: NSIndexPath) -> CustomerEntity {
        var key = self.activeKeys[indexPath.section]
        var array = self.objectsByKey[key]
        return array![indexPath.row]
    }
    
    func indexPathOfObject(object: CustomerEntity) -> NSIndexPath {
        
        var section: Int
        var row: Int
        
        // Get the first character of the last name or # if a number
        var key = object.lastName.substringToIndex(object.lastName.startIndex)
        if let number = self.formatter.numberFromString(key) {
            key = "#"
        }
        
        // Get the section number
        section = find(self.activeKeys, key)!
        
        // Get the row number
        let objectArray = self.objectsByKey[key]
        row = find(objectArray!, object)!
        
        var indexPath = NSIndexPath(forRow: row, inSection: section)
        
        return indexPath
        
    }
    
    // Tells the table view which section to scroll to when a particular key is
    // selected in the index. Since there aren’t always items for every key, need to
    // determine which section to scroll to when there are no items in the table view
    // for that key.
    func activeKeyIndexForKey(key: String) -> Int {
        
        var indexOfSelectedKey: Int
        // See if there are any items in the list for the selected key
        if let indexOfKey = find(self.activeKeys, key) {
            // There is, return it!
            return indexOfKey
        }
        
        // There isn't...Get the index of the selected key from the full list of keys
        indexOfSelectedKey = find(self.allKeys, key)!
        
        // Cycle through the list of keys for items in the list
        // Find the closest value below the selected key
        var indexOfCurrentKey: Int
        var previousIndex = 0
        for var i = 0; i < self.activeKeys.count; i++ {
            // Get the index of the key for items in the list
            indexOfCurrentKey = find(self.allKeys, self.activeKeys[i])!
            // If the index of the selected key is less
            if indexOfSelectedKey < indexOfCurrentKey {
                break
            }
            previousIndex = i
            
        }
        
        return previousIndex
    }
    
    func filterCustomersByName(searchText: String) -> NSMutableArray {
        
        var filteredList = NSMutableArray()
        
        for customer in self.entityList {
            
            let name = customer.firstName + " " + customer.lastName
            
            let result = name.compare(searchText, options: NSStringCompareOptions.CaseInsensitiveSearch, range: name.startIndex...name.endIndex, locale: nil)
            
            if result == NSComparisonResult.OrderedSame {
                filteredList.addObject(customer)
            }
        }
        
        return filteredList
    }
    
    func getCustomerByID(customerID: NSNumber) -> CustomerEntity? {
        
        var customer: CustomerEntity?
        
        let predicate = NSPredicate(format: "customerID = %d", Int(customerID))
        
        return customer
    }
    
}