//
//  MapViewController.swift
//  SimulatorDemo
//
//  Copyright (c) 2014 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit
import CoreLocation
import MapKit

class MapViewController: UIViewController, MKMapViewDelegate {

    @IBOutlet weak var mapView: MKMapView!
    
    var locationManager: CLLocationManager!
    var adjustRegion: MKCoordinateRegion?
    var externalScreen: UIScreen!
    var externalWindow: UIWindow!
    var externalMap: MKMapView?
    
    // Used for printer
    var printButton: UIBarButtonItem!
    
    override func viewDidLoad() {
        super.viewDidLoad()

        locationManager = CLLocationManager()
        locationManager.requestWhenInUseAuthorization()
        
        self.setupScreenConnectionNotificationHandlers()
        
        // Initialize an external screen if one is present
        let screens = UIScreen.screens()
        if screens.count > 1 {
            
            //An external screen is available. Get the first screen available
            self.initializeExternalScreen(screens[1] as! UIScreen)
        }
        
        // Add a print button to the navigation bar if printing is available
        if UIPrintInteractionController.isPrintingAvailable() {
            
            self.printButton = UIBarButtonItem(barButtonSystemItem: UIBarButtonSystemItem.Action, target: self, action: "printMapView")
            
            self.navigationItem.rightBarButtonItem = printButton;
        }
    }
    
    // Initialize an external screen
    func initializeExternalScreen(externalScreen: UIScreen) {
        
        self.externalScreen = externalScreen
        
        // Create a new window sized to the external screen's bounds
        self.externalWindow = UIWindow(frame: self.externalScreen.bounds)
        
        // Assign the screen object to the screen property of your new window
        self.externalWindow.screen = externalScreen;
        
        // Configure the window
        var view = UIView(frame: self.externalWindow.frame)
        view.backgroundColor = UIColor.whiteColor()
        self.externalMap = MKMapView(frame: view.frame)
        externalMap!.showsUserLocation = true
        
        view.addSubview(self.externalMap!)
        self.externalWindow.addSubview(view)
        
        // Make the window visible
        self.externalWindow.makeKeyAndVisible()
        
        self.adjustExternalScreenRegion()
    }
    
    func adjustExternalScreenRegion()
    {
        if let externalMap = self.externalMap {
            if let adjustRegion = self.adjustRegion {
                externalMap.setRegion(adjustRegion, animated: true)
            }
        }
    }
    
    // Set up screen connection notifications
    func setupScreenConnectionNotificationHandlers() {
        
        let center = NSNotificationCenter.defaultCenter()
        
        center.addObserver(self, selector: "handleScreenDidConnectNotification:", name: UIScreenDidConnectNotification, object: nil)
        
        center.addObserver(self, selector: "handleScreenDidDisconnectNotification:", name: UIScreenDidDisconnectNotification, object: nil)
    }
    
    // Handle external screen connection
    func handleScreenDidConnectNotification(aNotification: NSNotification) {
    
        if let screen = aNotification.object as? UIScreen {
            self.initializeExternalScreen(screen)
        }
    }
    
    // Handle external screen disconnection
    func handleScreenDidDisconnectNotification(aNotification: NSNotification) {
        
        if self.externalWindow != nil {
            
            self.externalWindow.hidden = true
            self.externalWindow = nil
        }
    }
    
    override func viewWillAppear(animated: Bool) {
        
        super.viewWillAppear(animated)
        self.mapView.showsUserLocation = false
        self.mapView.showsUserLocation = true
    }
    
    func mapView(mapView: MKMapView!, didUpdateUserLocation userLocation: MKUserLocation!) {
        
        // the location object that we want to initialize based on the string
        var location: CLLocationCoordinate2D = CLLocationCoordinate2D(latitude: userLocation.coordinate.latitude, longitude: userLocation.coordinate.longitude)
        
        var region = MKCoordinateRegionMakeWithDistance(location, 2000, 2000);
        
        // for correcting map aspect view
        self.adjustRegion = self.mapView.regionThatFits(region)
        self.mapView.setRegion(self.adjustRegion!, animated: true)
        
        region = MKCoordinateRegionMakeWithDistance(location, 100000, 100000);
        self.adjustRegion = self.mapView.regionThatFits(region)
        
        self.adjustExternalScreenRegion()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func printMapView() {
        
        // Create and configure a print info object
        let printInfo = UIPrintInfo(dictionary: nil)
        
        // Specify the output type and job name
        printInfo.outputType = UIPrintInfoOutputType.General
        printInfo.jobName = "Map Print"
        
        // Create and configure a print controller
        let printController = UIPrintInteractionController.sharedPrintController()!
        printController.printInfo = printInfo
        printController.showsPageRange = true
        var formatter = self.mapView.viewPrintFormatter()
        printController.printFormatter = formatter
        
        // Create a completion handler
        var completionHandler: UIPrintInteractionCompletionHandler = {(printController, completed, error) -> Void in
            
            if !completed {
                if error != nil {
                    println("Print failed: - domain: \(error.domain) error code: \(error.code)")
                }
            }
        }
        
        // Display the print controller
        if UIDevice.currentDevice().userInterfaceIdiom == .Pad {
            
            printController.presentFromBarButtonItem(printButton, animated: true, completionHandler: completionHandler)
        }
        else {
            printController.presentAnimated(true, completionHandler: completionHandler)
        }
    }

    @IBAction func done(sender: AnyObject) {
        
        self.dismissViewControllerAnimated(true, completion: nil)
    }
}
