//
//  InAppPurchaseManager.swift
//  TicTacToe
//
//  Created by Kevin McNeish on 6/9/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import Foundation
import StoreKit

public let IN_APP_PRODUCTS_RETRIEVED = "InAppProductsRetrieved"
public let IN_APP_PURCHASE_RESULT = "InAppPurchaseResult"
public let IN_APP_PURCHASE_RESTORE_ALL_RESULT = "InAppPurchaseRestoreAllTransactionsResult"
public let IN_APP_PURCHASE_RESTORE_RESULT = "InAppPurchaseRestoreTransactionResult"

private let singletonInstance = InAppPurchaseManager()

public class InAppPurchaseManager: NSObject, SKProductsRequestDelegate,
SKPaymentTransactionObserver {
    
    public var themeBundleList: Array<ThemeBundleEntity>?
    lazy var themeBundle = ThemeBundle<ThemeBundleEntity>()
    
    let numberFormatter = NSNumberFormatter()
    let buyNowString = NSLocalizedString("Buy now", comment: "")
    
    // Singleton sharedInstance
    public class var sharedInstance: InAppPurchaseManager {
        return singletonInstance
    }
    
    override init() {
        super.init()
        self.numberFormatter.numberStyle = NSNumberFormatterStyle.CurrencyStyle
    }
    
    // Call once at startup in AppDelegate
    public func observeTransactions() {
        SKPaymentQueue.defaultQueue().addTransactionObserver(self)
    }
    
    // Returns a boolean value indicating if the user can make payments
    public var canMakePurchases: Bool {
        return SKPaymentQueue.canMakePayments()
    }
    
    // Returns the localized price string for the specified product
    public func getLocalizedPriceStringForProduct(product: SKProduct) -> String {
        
        self.numberFormatter.locale = product.priceLocale
        return self.numberFormatter.stringFromNumber(product.price)!
    }
    
    // Purchase the specified product
    public func purchaseProduct(product: SKProduct) {
        let payment = SKPayment(product: product)
        SKPaymentQueue.defaultQueue().addPayment(payment)
    }
    
    // Request a list of products from the store
    public func requestProducts(productIDs: Array<String>) {
        
        // Create the product request object and send the request
        let productsRequest = SKProductsRequest(productIdentifiers: NSSet(array: productIDs) as Set<NSObject>)
        productsRequest.delegate = self
        productsRequest.start()
    }
    
    // Restore all of user's previous transactions
    public func restoreAllPurchases() {
        SKPaymentQueue.defaultQueue().restoreCompletedTransactions()
    }
    
    // Post a notification that purchase succeeded
    func postPurchaseSucceededNotification(transaction: SKPaymentTransaction) {
        
        let userInfo = ["success": true,
            "transaction": transaction]
        
        NSNotificationCenter.defaultCenter().postNotificationName(IN_APP_PURCHASE_RESULT, object: self, userInfo: userInfo)
    }
    
    // Post a notification that purchase failed
    public func postPurchaseFailedNotification(transaction: SKPaymentTransaction) {
        
        var userInfo: [NSObject: AnyObject] = ["success": false,
            "transaction": transaction]
        
        if let errorMessage = self.buildErrorMessage(transaction.error) {
            userInfo["errorMessage"] = errorMessage
        }
        
        NSNotificationCenter.defaultCenter().postNotificationName(IN_APP_PURCHASE_RESULT, object: self, userInfo: userInfo)
        
    }
    
    // Post a notification that purchase restored
    func postPurchaseRestoreSucceededNotification(transaction: SKPaymentTransaction) {
        
        let userInfo = ["success": true,
            "transaction": transaction]
        
        NSNotificationCenter.defaultCenter().postNotificationName(IN_APP_PURCHASE_RESTORE_RESULT, object: self, userInfo: userInfo)
    }
    
    // Builds an error message from the NSError object
    func buildErrorMessage(error: NSError) -> String? {
        
        var message: String?
        
        if error.code != SKErrorPaymentCancelled && error.code != SKErrorPaymentNotAllowed {
            
            switch error.code {
            case SKErrorUnknown, SKErrorClientInvalid:
                message = NSLocalizedString("This action is not allowed.", comment: "")
            case SKErrorPaymentInvalid:
                message = NSLocalizedString("Invalid Payment.", comment: "")
                break
            case SKErrorStoreProductNotAvailable:
                message = NSLocalizedString("Product not available.", comment: "")
                break
            default:
                break
            }
        }
        return message
    }
    
    // MARK: - SKProductsRequestDelegate
    // Handle the products request reponse
    public func productsRequest(request: SKProductsRequest!, didReceiveResponse response: SKProductsResponse!) {
        
        let validProducts = response.products as! Array<SKProduct>
        let invalidProducts = response.invalidProductIdentifiers as! Array<String>
        
        let userInfo = ["validProducts": validProducts,
            "invalidProducts": invalidProducts]
        
        NSNotificationCenter.defaultCenter().postNotificationName(IN_APP_PRODUCTS_RETRIEVED, object: self, userInfo: userInfo as [NSObject : AnyObject])
    }
    
    // MARK: - SKPaymentTransactionObserver
    public func paymentQueue(queue: SKPaymentQueue!, updatedTransactions transactions: [AnyObject]!) {
        
        for transaction in transactions as! [SKPaymentTransaction]! {
            
            switch transaction.transactionState {
                
            case .Purchasing:
                // Purchase is in progress
                break
                
            case .Purchased:
                // Purchase has completed
                queue.finishTransaction(transaction)
                let receiptURL = NSBundle.mainBundle().appStoreReceiptURL
                let receipt = NSData(contentsOfURL: receiptURL!)
                // TODO: Validate the receipt
                
                // Post a purchase notification. Client makes product available
                self.postPurchaseSucceededNotification(transaction)
                
            case .Deferred:
                // Waiting for parental consent. Let them continue to use the app
                break
                
            case .Failed:
                // Purchase failed
                queue.finishTransaction(transaction)
                self.postPurchaseFailedNotification(transaction)
                
            case .Restored:
                // Restored transaction
                queue.finishTransaction(transaction)
                
                // NOTE: We don't post a notification here because several
                // products may be restored. Notification is posted in the
                // paymentQueueRestoreCompletedTransactionsFinished or
                // restoreCompletedTransactionsFailedWithError methods instead.
                self.postPurchaseRestoreSucceededNotification(transaction)
            }
        }
    }
    
    // Restore Purchases succeeded
    public func paymentQueueRestoreCompletedTransactionsFinished(queue: SKPaymentQueue!) {
        
        let userInfo: [NSObject : AnyObject] = ["success": true,
            "count": queue.transactions.count]
        
        // Post a notification that restore all succeeded
        NSNotificationCenter.defaultCenter().postNotificationName(IN_APP_PURCHASE_RESTORE_ALL_RESULT, object: self, userInfo: userInfo)
    }
    
    // Restore Purchases failed
    public func paymentQueue(queue: SKPaymentQueue!, restoreCompletedTransactionsFailedWithError error: NSError!) {
        
        var userInfo: [NSObject: AnyObject] = ["success": false]
        if let errorMessage = self.buildErrorMessage(error) {
            userInfo["errorMessage"] = errorMessage
        }
        
        // Post a notification that restore purchases failed
        NSNotificationCenter.defaultCenter().postNotificationName(IN_APP_PURCHASE_RESTORE_ALL_RESULT, object: self, userInfo: userInfo)
    }
    
}