//
//  ThemeBundle.swift
//  TicTacToe
//
//  Created by Kevin McNeish on 4/29/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import Foundation
import StoreKit

public let THEMEBUNDLES_RETRIEVED = "ThemeBundlesRetrieved"
public let THEMEBUNDLE_PURCHASED = "ThemeBundlePurchased"
public let THEMEBUNDLE_ALL_PURCHASES_RESTORED = "ThemeBundleAllPurchasesRestored"

private let singletonInstance = ThemeBundle()

public class ThemeBundle<T: ThemeBundleEntity>: ABusinessObject<T> {
    
    public var unpurchasedBundleList: Array<ThemeBundleEntity>?
    let sortDescriptor = NSSortDescriptor(key: "displayOrder", ascending: true)
    var productsRetrievedNotificationToken: NSObjectProtocol?
    var inAppPurchaseNotificationToken: NSObjectProtocol?
    var inAppPurchaseRestoreNotificationToken: NSObjectProtocol?
    var inAppPurchaseRestoreAllNotificationToken: NSObjectProtocol?
    
    // Singleton sharedInstance
    public class var sharedInstance: ThemeBundle<ThemeBundleEntity> {
        return singletonInstance
    }
    
    public override init() {
        super.init()
    }
    
    deinit {
        
        if let token = self.productsRetrievedNotificationToken {
            NSNotificationCenter.defaultCenter().removeObserver(token)
        }
        if let token = self.inAppPurchaseNotificationToken {
            NSNotificationCenter.defaultCenter().removeObserver(token)
        }
        if let token = self.inAppPurchaseRestoreNotificationToken {
            NSNotificationCenter.defaultCenter().removeObserver(token)
        }
        if let token = self.inAppPurchaseRestoreAllNotificationToken {
            NSNotificationCenter.defaultCenter().removeObserver(token)
        }
    }
    
    // Gets all unpurchased, valid products from the App Store
    public func getUnpurchasedProductsFromAppStore() {
        
        // Get the list of unpurchased themes
        var tempBundleList: Array<ThemeBundleEntity> = self.getUnpurchasedThemeBundles()
        
        // Create an array of product IDs
        var productIDs = Array<String>()
        
        for themeBundle in tempBundleList {
            productIDs.append(themeBundle.themeBundleID)
        }
        
        // Register for notification of InAppPurchaseManager products retrieved
        self.productsRetrievedNotificationToken =
            NSNotificationCenter.defaultCenter().addObserverForName(IN_APP_PRODUCTS_RETRIEVED, object: nil, queue: NSOperationQueue.mainQueue()) { [unowned self] notification in
                
                NSNotificationCenter.defaultCenter().removeObserver(self.productsRetrievedNotificationToken!)
                
                let validProducts = notification.userInfo!["validProducts"] as! Array<SKProduct>
                let invalidProducts = notification.userInfo!["invalidProducts"] as! Array<String>
                
                for invalidProduct in invalidProducts {
                    
                    // // Remove the invalid product from the list
                    let themeBundleEntity = tempBundleList.filter{ $0.themeBundleID == invalidProduct}.first
                    
                    if themeBundleEntity != nil {
                        
                        let index = find(tempBundleList, themeBundleEntity!)
                        tempBundleList.removeAtIndex(index!)
                    }
                }
                
                for validProduct in validProducts {
                    // Store the product object on the entity
                    let themeBundleEntity = tempBundleList.filter{ $0.themeBundleID == validProduct.productIdentifier}.first!
                    themeBundleEntity.product = validProduct
                }
                
                self.unpurchasedBundleList = tempBundleList
                
                let userInfo = ["unpurchasedBundleList": self.unpurchasedBundleList!]
                
                NSNotificationCenter.defaultCenter().postNotificationName(THEMEBUNDLES_RETRIEVED, object: self, userInfo: userInfo as [NSObject : AnyObject])
        }
        
        // Get Color Theme products from App Store
        InAppPurchaseManager.sharedInstance.requestProducts(productIDs)
    }
    
    // Retrieves all unpurchased themes
    func getUnpurchasedThemeBundles() -> Array<T> {
        
        let predicate = NSPredicate(format: "purchased == %@",NSNumber(bool: false))
        return self.getEntitiesSortedBy(self.sortDescriptor, matchingPredicate: predicate)
    }
    
    // Retrieves all purchased themes
    public func getPurchasedThemeBundles() -> Array<T> {
        
        let predicate = NSPredicate(format: "purchased == %@",NSNumber(bool: true))
        return self.getEntitiesSortedBy(self.sortDescriptor, matchingPredicate: predicate)
    }
    
    // Gets the ThemeBundleEntity by theme bundle ID
    public func getThemeBundleByID(themeBundleID: String) -> ThemeBundleEntity {
        let predicate = NSPredicate(format: "themeBundleID == %@",themeBundleID)
        return self.getEntitiesSortedBy(self.sortDescriptor, matchingPredicate: predicate).first!
    }
    
    // Mark the specified theme bundle as purchased
    public func markBundleAsPurchased(themeBundleID: String) {
        
        let themeBundleEntity = self.getThemeBundleByID(themeBundleID)
        themeBundleEntity.purchased = NSNumber(bool: true)
        self.saveEntities()
    }
    
    // Purchase the specified product
    public func purchaseProduct(product: SKProduct) {
        
        // Register for notification of InAppPurchaseManager purchased color theme
        self.inAppPurchaseNotificationToken =
            NSNotificationCenter.defaultCenter().addObserverForName(IN_APP_PURCHASE_RESULT, object: nil, queue: NSOperationQueue.mainQueue()) { [unowned self] notification in
                
                NSNotificationCenter.defaultCenter().removeObserver(self.inAppPurchaseNotificationToken!)
                
                let success = notification.userInfo!["success"] as! Bool
                let transaction = notification.userInfo!["transaction"] as! SKPaymentTransaction
                
                // If transaction succeeded, make the product available
                if success {
                    
                    let productID = transaction.payment.productIdentifier
                    self.markBundleAsPurchased(productID)
                }
                
                // Post a notification that a theme bundle was purchased,
                // passing the original userInfo object through
                NSNotificationCenter.defaultCenter().postNotificationName(THEMEBUNDLE_PURCHASED, object: self, userInfo: notification.userInfo)
        }
        
        InAppPurchaseManager.sharedInstance.purchaseProduct(product)
    }
    
    // Restore in-app Purchase
    public func restoreAllInAppPurchases() {
        
        // Register for notification of individual restored purchases
        self.inAppPurchaseRestoreNotificationToken =
            NSNotificationCenter.defaultCenter().addObserverForName(IN_APP_PURCHASE_RESTORE_RESULT, object: nil, queue: NSOperationQueue.mainQueue()) { [unowned self] notification in
                
                NSNotificationCenter.defaultCenter().removeObserver(self.inAppPurchaseRestoreNotificationToken!)
                
                let success = notification.userInfo!["success"] as! Bool
                let transaction = notification.userInfo!["transaction"] as! SKPaymentTransaction
                
                // If transaction succeeded, make the product available
                if success {
                    
                    let productID = transaction.payment.productIdentifier
                    self.markBundleAsPurchased(productID)
                }
        }
        
        // Register for notification of ALL restored purchases
        self.inAppPurchaseRestoreAllNotificationToken =
            NSNotificationCenter.defaultCenter().addObserverForName(IN_APP_PURCHASE_RESTORE_ALL_RESULT, object: nil, queue: NSOperationQueue.mainQueue()) { [unowned self] notification in
                
                // Remove the observer
                NSNotificationCenter.defaultCenter().removeObserver(self.inAppPurchaseRestoreAllNotificationToken!)
                
                // Post a notification that all purchases were restored,
                // passing the original userInfo object through
                NSNotificationCenter.defaultCenter().postNotificationName(THEMEBUNDLE_ALL_PURCHASES_RESTORED, object: self, userInfo: notification.userInfo)
        }
        
        // Restore all purchases
        InAppPurchaseManager.sharedInstance.restoreAllPurchases()
    }
    
}