//
//  mmBusinessObject.swift
//  mmiOS8
//
//  Created by Kevin McNeish on 9/9/14.
//  Copyright (c) 2014 Oak Leaf Enterprises, Inc. All rights reserved.
//
// 11/19/14
// Changed lightweight migration so it's always on

import Foundation
import CoreData

public enum SaveState {
    case Error, RulesBroken, SaveComplete
}

public class mmBusinessObject<T: NSManagedObject>
{
    var dbName : String = ""
    var copyDatabaseIfNotPresent : Bool = false
    var error : NSError?
    
    init()
    {
    }
    
    // The directory the application uses to store the Core Data store file. This code uses a directory named "com.oakleafsd.<ProjectName>" in the application's documents Application Support directory.
    lazy var applicationDocumentsDirectory: NSURL = {
        let urls = NSFileManager.defaultManager().URLsForDirectory(.DocumentDirectory, inDomains: .UserDomainMask)
        
        println(urls);
        
        return urls[urls.count-1] as! NSURL
        }()
    
    // Returns the managed object context for the application (which is already bound to the persistent store coordinator for the application.) This property is optional since there are legitimate error conditions that could cause the creation of the context to fail.
    lazy var managedObjectContext: NSManagedObjectContext? = {
        
        let coordinator = self.persistentStoreCoordinator
        if coordinator == nil {
            return nil
        }
        var managedObjectContext = NSManagedObjectContext()
        managedObjectContext.persistentStoreCoordinator = coordinator
        return managedObjectContext
        }()
    
    // The managed object model for the application. This property is not optional. It is a fatal error for the application not to be able to find and load its model.
    lazy var managedObjectModel: NSManagedObjectModel = {
        
        let modelURL = NSBundle.mainBundle().URLForResource(self.dbName, withExtension: "momd")!
        var model: NSManagedObjectModel? = NSManagedObjectModel(contentsOfURL: modelURL)
        return model!
        }()
    
    // The persistent store coordinator for the application. This implementation creates and return a coordinator, having added the store for the application to it. This property is optional since there are legitimate error conditions that could cause the creation of the store to fail.
    lazy var persistentStoreCoordinator: NSPersistentStoreCoordinator? = {
        
        let url : NSURL = self.getStoreURL()
        
        if self.copyDatabaseIfNotPresent
        {
            //            // Get the Documents directory
            //            var paths = NSSearchPathForDirectoriesInDomains(NSSearchPathDirectory.DocumentDirectory, NSSearchPathDomainMask.UserDomainMask, true)
            //            var docsDir = paths[0] as! String
            //
            //            // Append the database name to the path
            //            var dbcPath = docsDir.stringByAppendingPathComponent(self.dbName + ".sqlite")
            
            let dbcPath = url.path!
            
            var fileManager = NSFileManager.defaultManager()
            
            // Check if the sqlite database already exists
            if !fileManager.fileExistsAtPath(dbcPath)
            {
                // It doesn't. Check if a .sqlite database was distributed with this app
                var defaultStorePath = NSBundle.mainBundle().pathForResource(self.dbName, ofType: "sqlite")
                if defaultStorePath != nil
                {
                    // It was, copy the embedded database to the app's Documents directory
                    fileManager.copyItemAtPath(defaultStorePath!, toPath: dbcPath, error: nil)
                }
            }
        }
        
        var error: NSError? = nil
        
        // Create the coordinator and store
        var coordinator: NSPersistentStoreCoordinator? = NSPersistentStoreCoordinator(managedObjectModel: self.managedObjectModel)
        
        var failureReason = "There was an error creating or loading the application's saved data."
        
        if coordinator!.addPersistentStoreWithType(NSSQLiteStoreType, configuration: nil,
            URL: url,
            options: [NSMigratePersistentStoresAutomaticallyOption: true,
                NSInferMappingModelAutomaticallyOption: true],
            error: &error) == nil {
                coordinator = nil
                // Report any error we got.
                let dict = NSMutableDictionary()
                dict[NSLocalizedDescriptionKey] = "Failed to initialize the application's saved data"
                dict[NSLocalizedFailureReasonErrorKey] = failureReason
                dict[NSUnderlyingErrorKey] = error
                error = NSError(domain: "YOUR_ERROR_DOMAIN", code: 9999, userInfo: dict as [NSObject : AnyObject])
                NSLog("Unresolved error \(error), \(error!.userInfo)")
                abort()
        }
        
        return coordinator
        }()
    
    func getStoreURL() -> NSURL {
        return self.applicationDocumentsDirectory.URLByAppendingPathComponent("\(self.dbName).sqlite")
    }
    
    // Saves changes to all entities
    public func saveEntities() -> (state: SaveState, message: String?)
    {
        var saveState: SaveState
        var saveMessage: String?
        
        // Business rules passed
        if let managedObjectContext = self.managedObjectContext
        {
            if managedObjectContext.hasChanges && !managedObjectContext.save(&error)
            {
                saveState = SaveState.Error
                saveMessage = error?.description
            }
            else
            {
                saveState = SaveState.SaveComplete
                saveMessage = nil
            }
        }
        else
        {
            saveState = SaveState.Error
            saveMessage = "Database error"
        }
        
        return (saveState, saveMessage)
    }
    
    // Register a related business controller object
    // This causes them to use the same object context
    func registerChildObject(controllerObject: mmBusinessObject)
    {
        controllerObject.managedObjectContext = self.managedObjectContext
    }
    
    // Gets the entity class name string from <T>
    var entityClassName : String {
        var className = NSStringFromClass(T)
        if className.rangeOfString(".") != nil {
            className = className.pathExtension
        }
        return className
    }
    
    // Gets all entities of the default type
    public func getAllEntities() -> Array<T>
    {
        return self.getEntities(sortedBy: nil, matchingPredicate: nil)
    }
    
    // Get all entities sorted by the descriptor and matching the predicate
    func getEntities(sortedBy sortDescriptor:NSSortDescriptor?, matchingPredicate predicate:NSPredicate?) -> Array<T>
    {
        var error: NSError?
        
        // Create the request object
        var request : NSFetchRequest = NSFetchRequest()
        
        // Set the entity type to be fetched
        var entityDescription : NSEntityDescription! = NSEntityDescription.entityForName(self.entityClassName, inManagedObjectContext: self.managedObjectContext!)
        
        request.entity = entityDescription
        
        if predicate != nil
        {
            request.predicate = predicate
        }
        
        if sortDescriptor != nil
        {
            request.sortDescriptors = [sortDescriptor] as Array<NSSortDescriptor!>
        }
        
        // Execute the fetch
        if let results = self.managedObjectContext?.executeFetchRequest(request, error: &error) as! Array<T>? {
            return results
        }
        else {
            return Array<T>() // return an empty array
        }
    }
    
    // Gets all entities sorted by descriptor
    func getAllEntitiesSortedBy(sortDescriptor:NSSortDescriptor) -> Array<T>
    {
        return self.getEntities(sortedBy: sortDescriptor, matchingPredicate: nil)
    }
    
    // Get entities of the default type matching the predicate
    func getEntitiesMatchingPredicate(predicate: NSPredicate) -> Array<T>
    {
        return self.getEntities(sortedBy: nil, matchingPredicate: predicate)
    }
    
    // Get entities of the default type sorted by descriptor and matching the predicate
    func getEntitiesSortedBy(sortDescriptor: NSSortDescriptor, matchingPredicate predicate:NSPredicate) -> Array<T>?
    {
        return self.getEntities(sortedBy: sortDescriptor, matchingPredicate: predicate)
    }
    
    // Save changes to the specified entity
    func saveEntity(entity: T) -> (state: SaveState, message: String?)
    {
        var saveState: SaveState
        var saveMessage: String?
        
        // Check the business rules
        //saveMessage = entity.checkRules()
        
        if saveMessage == nil
        {
            let result = self.saveEntities()
            saveState = result.state
            saveMessage = result.message
        }
        else
        {
            // Business rules failed
            saveState = SaveState.RulesBroken
        }
        return (state: saveState, message: saveMessage)
    }
    
    // Adds the list of entities to this business object context
    func addEntitiesToObjectContext(entityList: Array<T>)
    {
        for entityOriginal: T in entityList
        {
            let keys = Array(entityOriginal.entity.attributesByName.keys)
            let dictionary = entityOriginal.dictionaryWithValuesForKeys(keys)
            var entityCopy = self.createEntity()
            entityCopy.setValuesForKeysWithDictionary(dictionary)
        }
    }
    
    // Create a new entity of the default type
    func createEntity() -> T
    {
        return NSEntityDescription.insertNewObjectForEntityForName(self.entityClassName, inManagedObjectContext: self.managedObjectContext!) as! T
    }
    
    // Mark the specified entity for deletion
    func deleteEntity(entity: T)
    {
        self.managedObjectContext?.deleteObject(entity)
    }
    
}
